@@ -15,7 +15,7 @@
  */
 
 /**
- * @TODO: multi threaded compiling of the same class but with different roots
+ * @todo multi threaded compiling of the same class but with different roots
  * for compilation... T1 compiles A, which uses B, T2 compiles B... mark A and B
  * as parsed and then synchronize compilation. Problems: How to synchronize? 
  * How to get error messages?   
@@ -32,10 +32,7 @@ import org.objectweb.asm.ClassWriter;
 
 import java.io.*;
 import java.net.*;
-import java.security.AccessController;
-import java.security.CodeSource;
-import java.security.PrivilegedAction;
-import java.security.ProtectionDomain;
+import java.security.*;
 import java.util.*;
 
 /**
@@ -52,7 +49,7 @@ import java.util.*;
  * @author Bing Ran
  * @author <a href="mailto:scottstirling@rcn.com">Scott Stirling</a>
  * @author <a href="mailto:blackdrag@gmx.org">Jochen Theodorou</a>
- * @version $Revision: 10686 $
+ * @version $Revision: 15765 $
  */
 public class GroovyClassLoader extends URLClassLoader {
 
@@ -60,6 +57,10 @@ public class GroovyClassLoader extends URLClassLoader {
      * this cache contains the loaded classes or PARSING, if the class is currently parsed
      */
     protected final Map classCache = new HashMap();
+    /**
+     * this cache contains mappings of file name to class. It is used
+     * to bypass compilation.
+     */
     protected final Map sourceCache = new HashMap();
     private final CompilerConfiguration config;
     private Boolean recompile;
@@ -103,7 +104,7 @@ public class GroovyClassLoader extends URLClassLoader {
     /**
      * creates a GroovyClassLaoder.
      *
-     * @param parent                    the parten class loader
+     * @param parent                    the parent class loader
      * @param config                    the compiler configuration
      * @param useConfigurationClasspath determines if the configurations classpath should be added
      */
@@ -256,7 +257,7 @@ public class GroovyClassLoader extends URLClassLoader {
      * @return the main class defined in the given script
      */
     public Class parseClass(GroovyCodeSource codeSource, boolean shouldCacheSource) throws CompilationFailedException {
-        synchronized (this) {
+        synchronized (sourceCache) {
             Class answer = (Class) sourceCache.get(codeSource.getName());
             if (answer != null) return answer;
 
@@ -323,6 +324,28 @@ public class GroovyClassLoader extends URLClassLoader {
         throw new DeprecationException("the method groovy.lang.GroovyClassLoader#defineClass(String,byte[],ProtectionDomain) is no longer used internally and removed");
     }
 
+    protected PermissionCollection getPermissions(CodeSource codeSource) {
+        PermissionCollection perms;
+        try {
+            perms = super.getPermissions(codeSource);
+        } catch (SecurityException e) {
+            // We lied about our CodeSource and that makes URLClassLoader unhappy.
+            perms = new Permissions();
+        }
+
+        ProtectionDomain myDomain = AccessController.doPrivileged(new PrivilegedAction<ProtectionDomain>() {
+            public ProtectionDomain run() {
+                return getClass().getProtectionDomain();
+            }
+        });
+        PermissionCollection myPerms = myDomain.getPermissions();
+        for (Enumeration<Permission> elements = myPerms.elements(); elements.hasMoreElements();) {
+            perms.add(elements.nextElement());
+        }
+        perms.setReadOnly();
+        return perms;
+    }
+
     public static class InnerLoader extends GroovyClassLoader {
         private final GroovyClassLoader delegate;
         private final long timeStamp;
@@ -443,7 +466,6 @@ public class GroovyClassLoader extends URLClassLoader {
         protected Class createClass(byte[] code, ClassNode classNode) {
             GroovyClassLoader cl = getDefiningClassLoader();
             Class theClass = cl.defineClass(classNode.getName(), code, 0, code.length, unit.getAST().getCodeSource());
-            //cl.resolveClass(theClass);
             this.loadedClasses.add(theClass);
 
             if (generatedClass == null) {
@@ -564,6 +586,7 @@ public class GroovyClassLoader extends URLClassLoader {
      */
     protected boolean isRecompilable(Class cls) {
         if (cls == null) return true;
+        if (cls.getClassLoader()==this) return false;
         if (recompile == null && !config.getRecompileGroovySource()) return false;
         if (recompile != null && !recompile.booleanValue()) return false;
         if (!GroovyObject.class.isAssignableFrom(cls)) return false;
@@ -617,16 +640,6 @@ public class GroovyClassLoader extends URLClassLoader {
         boolean recompile = isRecompilable(cls);
         if (!recompile) return cls;
 
-        // check security manager
-        SecurityManager sm = System.getSecurityManager();
-        if (sm != null) {
-            String className = name.replace('/', '.');
-            int i = className.lastIndexOf('.');
-            if (i != -1) {
-                sm.checkPackageAccess(className.substring(0, i));
-            }
-        }
-
         // try parent loader
         ClassNotFoundException last = null;
         try {
@@ -642,18 +655,29 @@ public class GroovyClassLoader extends URLClassLoader {
                 throw ncdfe;
             }
         }
-
-        if (cls != null) {
-            // prefer class if no recompilation
-            preferClassOverScript |= !recompile;
-            if (preferClassOverScript) return cls;
+        
+        // check security manager
+        SecurityManager sm = System.getSecurityManager();
+        if (sm != null) {
+            String className = name.replace('/', '.');
+            int i = className.lastIndexOf('.');
+            // no checks on the sun.reflect classes for reflection speed-up
+            // in particular ConstructorAccessorImpl, MethodAccessorImpl, FieldAccessorImpl and SerializationConstructorAccessorImpl
+            // which are generated at runtime by the JDK
+            if (i != -1 && !className.startsWith("sun.reflect.")) {
+                sm.checkPackageAccess(className.substring(0, i));
+            }
         }
 
+        // prefer class if no recompilation
+        if (cls != null && preferClassOverScript) return cls;
+
         // at this point the loading from a parent loader failed
         // and we want to recompile if needed.
         if (lookupScriptFiles) {
-            // synchronize on this, as we want only one compilation at the same time
-            synchronized (this) {
+            // synchronize on sourceCache, as we want only one 
+            // compilation at the same time
+            synchronized (sourceCache) {
                 // try groovy file
                 try {
                     // check if recompilation already happened.
@@ -715,7 +739,7 @@ public class GroovyClassLoader extends URLClassLoader {
      * @see java.lang.ClassLoader#loadClass(java.lang.String, boolean)
      */
     protected Class loadClass(final String name, boolean resolve) throws ClassNotFoundException {
-        return loadClass(name, true, false, resolve);
+        return loadClass(name, true, true, resolve);
     }
 
     /**
@@ -858,5 +882,8 @@ public class GroovyClassLoader extends URLClassLoader {
         synchronized (classCache) {
             classCache.clear();
         }
+        synchronized (sourceCache) {
+            sourceCache.clear();
+        }
     }
 }
