@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2009 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,7 +26,6 @@ import java.sql.Connection;
 import java.sql.DriverManager;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
-import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.sql.Types;
@@ -38,15 +37,22 @@ import java.util.regex.Pattern;
 
 import javax.sql.DataSource;
 
+import org.codehaus.groovy.runtime.DefaultGroovyMethods;
+
 /**
  * Represents an extent of objects
  *
  * @author Chris Stevenson
- * @author <a href="mailto:james@coredevelopers.net">James Strachan </a>
- * @version $Revision: 10922 $
+ * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
+ * @author Paul King
+ * @author Marc DeXeT
+ * @version $Revision: 16362 $
  */
 public class Sql {
 
+    /**
+     * Hook to allow derived classes to access the log
+     */
     protected Logger log = Logger.getLogger(getClass().getName());
 
     private DataSource dataSource;
@@ -57,19 +63,29 @@ public class Sql {
     private int resultSetConcurrency = ResultSet.CONCUR_READ_ONLY;
     private int resultSetHoldability = -1;
 
-    /**
-     * let's only warn of using deprecated methods once
-     */
-    private boolean warned;
-
     // store the last row count for executeUpdate
-    int updateCount = 0;
+    private int updateCount = 0;
 
-    /**
+    /*
      * allows a closure to be used to configure the statement before its use
      */
     private Closure configureStatement;
 
+    /*
+     * property for allowing connection caching feature
+     */
+    private boolean cacheConnection;
+
+    /*
+     * property for allowing statements caching feature
+     */
+    private boolean cacheStatements;
+
+    /*
+     * Statement cache
+     */
+    private final Map<String, Statement> statementCache = new HashMap<String, Statement>();
+
     /**
      * Creates a new Sql instance given a JDBC connection URL.
      *
@@ -177,6 +193,7 @@ public class Sql {
      * Gets the resultSetType for statements created using the connection.
      *
      * @return the current resultSetType value
+     * @since 1.5.2
      */
     public int getResultSetType() {
         return resultSetType;
@@ -192,6 +209,7 @@ public class Sql {
      *         <code>ResultSet.TYPE_FORWARD_ONLY</code>,
      *         <code>ResultSet.TYPE_SCROLL_INSENSITIVE</code>, or
      *         <code>ResultSet.TYPE_SCROLL_SENSITIVE</code>
+     * @since 1.5.2
      */
     public void setResultSetType(int resultSetType) {
         this.resultSetType = resultSetType;
@@ -201,6 +219,7 @@ public class Sql {
      * Gets the resultSetConcurrency for statements created using the connection.
      *
      * @return the current resultSetConcurrency value
+     * @since 1.5.2
      */
     public int getResultSetConcurrency() {
         return resultSetConcurrency;
@@ -215,6 +234,7 @@ public class Sql {
      *        constants:
      *         <code>ResultSet.CONCUR_READ_ONLY</code> or
      *         <code>ResultSet.CONCUR_UPDATABLE</code>
+     * @since 1.5.2
      */
     public void setResultSetConcurrency(int resultSetConcurrency) {
         this.resultSetConcurrency = resultSetConcurrency;
@@ -224,6 +244,7 @@ public class Sql {
      * Gets the resultSetHoldability for statements created using the connection.
      *
      * @return the current resultSetHoldability value or -1 if not set
+     * @since 1.5.2
      */
     public int getResultSetHoldability() {
         return resultSetHoldability;
@@ -238,6 +259,7 @@ public class Sql {
      *        constants:
      *         <code>ResultSet.HOLD_CURSORS_OVER_COMMIT</code> or
      *         <code>ResultSet.CLOSE_CURSORS_AT_COMMIT</code>
+     * @since 1.5.2
      */
     public void setResultSetHoldability(int resultSetHoldability) {
         this.resultSetHoldability = resultSetHoldability;
@@ -429,7 +451,7 @@ public class Sql {
      * Constructs an SQL instance using the given Connection. It is the caller's
      * responsibility to close the Connection after the Sql instance has been
      * used. You can do this on the connection object directly or by calling the
-     * {@link Connection#close()}  method.
+     * {@link Connection#close()} method.
      *
      * @param connection the Connection to use
      */
@@ -462,7 +484,7 @@ public class Sql {
      */
     public void query(String sql, Closure closure) throws SQLException {
         Connection connection = createConnection();
-        Statement statement = createConnection(connection);
+        Statement statement = getStatement(connection, sql);
         configure(statement);
         ResultSet results = null;
         try {
@@ -479,7 +501,7 @@ public class Sql {
         }
     }
 
-    private Statement createConnection(Connection connection) throws SQLException {
+    private Statement createStatement(Connection connection) throws SQLException {
         if (resultSetHoldability == -1) {
             return connection.createStatement(resultSetType, resultSetConcurrency);
         }
@@ -501,7 +523,7 @@ public class Sql {
         ResultSet results = null;
         try {
             log.fine(sql);
-            statement = connection.prepareStatement(sql);
+            statement = getPreparedStatement(connection, sql);
             setParameters(params, statement);
             configure(statement);
             results = statement.executeQuery();
@@ -520,7 +542,7 @@ public class Sql {
      * Performs the given SQL query calling the closure with the result set.
      *
      * @param gstring a GString containing the SQL query with embedded params
-     * @param closure  called for each row with a GroovyResultSet
+     * @param closure called for each row with a GroovyResultSet
      * @throws SQLException if a database access error occurs
      */
     public void query(GString gstring, Closure closure) throws SQLException {
@@ -533,8 +555,8 @@ public class Sql {
      * Performs the given SQL query calling the closure with each row of the
      * result set.
      *
-     * @param sql      the sql statement
-     * @param closure  called for each row with a GroovyResultSet
+     * @param sql     the sql statement
+     * @param closure called for each row with a GroovyResultSet
      * @throws SQLException if a database access error occurs
      */
     public void eachRow(String sql, Closure closure) throws SQLException {
@@ -551,7 +573,7 @@ public class Sql {
      */
     public void eachRow(String sql, Closure metaClosure, Closure rowClosure) throws SQLException {
         Connection connection = createConnection();
-        Statement statement = createConnection(connection);
+        Statement statement = getStatement(connection, sql);
         configure(statement);
         ResultSet results = null;
         try {
@@ -586,7 +608,7 @@ public class Sql {
         ResultSet results = null;
         try {
             log.fine(sql);
-            statement = connection.prepareStatement(sql);
+            statement = getPreparedStatement(connection, sql);
             setParameters(params, statement);
             configure(statement);
             results = statement.executeQuery();
@@ -618,30 +640,6 @@ public class Sql {
         eachRow(sql, params, closure);
     }
 
-    /**
-     * @deprecated please use eachRow instead
-     */
-    public void queryEach(String sql, Closure closure) throws SQLException {
-        warnDeprecated();
-        eachRow(sql, closure);
-    }
-
-    /**
-     * @deprecated please use eachRow instead
-     */
-    public void queryEach(String sql, List params, Closure closure) throws SQLException {
-        warnDeprecated();
-        eachRow(sql, params, closure);
-    }
-
-    /**
-     * @deprecated please use eachRow instead
-     */
-    public void queryEach(GString gstring, Closure closure) throws SQLException {
-        warnDeprecated();
-        eachRow(gstring, closure);
-    }
-
     /**
      * Performs the given SQL query and return the rows of the result set.
      *
@@ -675,9 +673,9 @@ public class Sql {
      * @throws SQLException if a database access error occurs
      */
     public List rows(String sql, Closure metaClosure) throws SQLException {
-        List results = new ArrayList();
+        List<GroovyRowResult> results = new ArrayList<GroovyRowResult>();
         Connection connection = createConnection();
-        Statement statement = createConnection(connection);
+        Statement statement = getStatement(connection, sql);
         configure(statement);
         ResultSet rs = null;
         try {
@@ -686,13 +684,7 @@ public class Sql {
             if (metaClosure != null) metaClosure.call(rs.getMetaData());
 
             while (rs.next()) {
-                ResultSetMetaData metadata = rs.getMetaData();
-                LinkedHashMap lhm = new LinkedHashMap(metadata.getColumnCount(), 1);
-                for (int i = 1; i <= metadata.getColumnCount(); i++) {
-                    lhm.put(metadata.getColumnName(i), rs.getObject(i));
-                }
-                GroovyRowResult row = new GroovyRowResult(lhm);
-                results.add(row);
+                results.add(DefaultGroovyMethods.toRowResult(rs));
             }
             return (results);
         } catch (SQLException e) {
@@ -713,24 +705,18 @@ public class Sql {
      * @throws SQLException if a database access error occurs
      */
     public List rows(String sql, List params) throws SQLException {
-        List results = new ArrayList();
+        List<GroovyRowResult> results = new ArrayList<GroovyRowResult>();
         Connection connection = createConnection();
         PreparedStatement statement = null;
         ResultSet rs = null;
         try {
             log.fine(sql);
-            statement = connection.prepareStatement(sql);
+            statement = getPreparedStatement(connection, sql);
             setParameters(params, statement);
             configure(statement);
             rs = statement.executeQuery();
             while (rs.next()) {
-                ResultSetMetaData metadata = rs.getMetaData();
-                LinkedHashMap lhm = new LinkedHashMap(metadata.getColumnCount(), 1);
-                for (int i = 1; i <= metadata.getColumnCount(); i++) {
-                    lhm.put(metadata.getColumnName(i), rs.getObject(i));
-                }
-                GroovyRowResult row = new GroovyRowResult(lhm);
-                results.add(row);
+                results.add(DefaultGroovyMethods.toRowResult(rs));
             }
             return (results);
         }
@@ -788,7 +774,7 @@ public class Sql {
     /**
      * Executes the given piece of SQL.
      *
-     * @param sql    the SQL statement
+     * @param sql the SQL to execute
      * @return <code>true</code> if the first result is a <code>ResultSet</code>
      *         object; <code>false</code> if it is an update count or there are
      *         no results
@@ -799,7 +785,7 @@ public class Sql {
         Statement statement = null;
         try {
             log.fine(sql);
-            statement = createConnection(connection);
+            statement = getStatement(connection, sql);
             configure(statement);
             boolean isResultSet = statement.execute(sql);
             this.updateCount = statement.getUpdateCount();
@@ -829,7 +815,7 @@ public class Sql {
         PreparedStatement statement = null;
         try {
             log.fine(sql);
-            statement = connection.prepareStatement(sql);
+            statement = getPreparedStatement(connection, sql);
             setParameters(params, statement);
             configure(statement);
             boolean isResultSet = statement.execute();
@@ -874,13 +860,13 @@ public class Sql {
         Statement statement = null;
         try {
             log.fine(sql);
-            statement = createConnection(connection);
+            statement = getStatement(connection, sql);
             configure(statement);
             boolean hasResultSet = statement.execute(sql, Statement.RETURN_GENERATED_KEYS);
 
             // Prepare a list to contain the auto-generated column
             // values, and then fetch them from the statement.
-            List autoKeys = new ArrayList();
+            List<List> autoKeys = new ArrayList<List>();
             ResultSet keys = statement.getGeneratedKeys();
             int count = keys.getMetaData().getColumnCount();
 
@@ -929,14 +915,14 @@ public class Sql {
             log.fine(sql);
 
             // Prepare a statement for the SQL and then execute it.
-            statement = connection.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
+            statement = getPreparedStatement(connection, sql, Statement.RETURN_GENERATED_KEYS);
             setParameters(params, statement);
             configure(statement);
             this.updateCount = statement.executeUpdate();
 
             // Prepare a list to contain the auto-generated column
             // values, and then fetch them from the statement.
-            List autoKeys = new ArrayList();
+            List<List> autoKeys = new ArrayList<List>();
             ResultSet keys = statement.getGeneratedKeys();
             int count = keys.getMetaData().getColumnCount();
 
@@ -1005,7 +991,7 @@ public class Sql {
     /**
      * Executes the given SQL update.
      *
-     * @param sql    the SQL statement
+     * @param sql the SQL to execute
      * @return the number of rows updated or 0 for SQL statements that return nothing
      * @throws SQLException if a database access error occurs
      */
@@ -1014,7 +1000,7 @@ public class Sql {
         Statement statement = null;
         try {
             log.fine(sql);
-            statement = createConnection(connection);
+            statement = getStatement(connection, sql);
             configure(statement);
             this.updateCount = statement.executeUpdate(sql);
             return this.updateCount;
@@ -1041,7 +1027,7 @@ public class Sql {
         PreparedStatement statement = null;
         try {
             log.fine(sql);
-            statement = connection.prepareStatement(sql);
+            statement = getPreparedStatement(connection, sql);
             setParameters(params, statement);
             configure(statement);
             this.updateCount = statement.executeUpdate();
@@ -1072,7 +1058,7 @@ public class Sql {
     /**
      * Performs a stored procedure call.
      *
-     * @param sql    the SQL statement
+     * @param sql the SQL statement
      * @return the number of rows updated or 0 for SQL statements that return nothing
      * @throws SQLException if a database access error occurs
      */
@@ -1125,8 +1111,7 @@ public class Sql {
             List results = new ArrayList();
             int indx = 0;
             int inouts = 0;
-            for (Iterator iter = params.iterator(); iter.hasNext();) {
-                Object value = iter.next();
+            for (Object value : params) {
                 if (value instanceof OutParameter) {
                     if (value instanceof ResultSetOutParameter) {
                         results.add(CallResultSet.getImpl(statement, indx));
@@ -1164,13 +1149,12 @@ public class Sql {
         return call(sql, params);
     }
 
-
     /**
      * Performs a stored procedure call with the given parameters,
      * calling the closure once with all result objects.
      *
      * @param gstring a GString containing the SQL query with embedded params
-     * @param closure  called for each row with a GroovyResultSet
+     * @param closure called for each row with a GroovyResultSet
      * @throws SQLException if a database access error occurs
      */
     public void call(GString gstring, Closure closure) throws Exception {
@@ -1187,9 +1171,17 @@ public class Sql {
      * @throws SQLException if a database access error occurs
      */
     public void close() throws SQLException {
-        if (useConnection != null) {
+        if (useConnection == null) {
+            log.log(Level.FINEST, "Close operation not supported when using datasets - attempt to close ignored");
+            return;
+        }
+        try {
             useConnection.close();
         }
+        catch (SQLException e) {
+            log.log(Level.SEVERE, "Caught exception closing connection: " + e, e);
+            throw e;
+        }
     }
 
     public DataSource getDataSource() {
@@ -1197,21 +1189,45 @@ public class Sql {
     }
 
 
-    public void commit() {
+    /**
+     * If this SQL object was created with a Connection then this method commits
+     * the connection. If this SQL object was created from a DataSource then
+     * this method does nothing.
+     *
+     * @throws SQLException if a database access error occurs
+     */
+    public void commit() throws SQLException {
+        if (useConnection == null) {
+            log.log(Level.FINEST, "Commit operation not supported when using datasets unless using withTransaction or cacheConnection - attempt to commit ignored");
+            return;
+        }
         try {
-            this.useConnection.commit();
+            useConnection.commit();
         }
         catch (SQLException e) {
-            log.log(Level.SEVERE, "Caught exception commiting connection: " + e, e);
+            log.log(Level.SEVERE, "Caught exception committing connection: " + e, e);
+            throw e;
         }
     }
 
-    public void rollback() {
+    /**
+     * If this SQL object was created with a Connection then this method rolls back
+     * the connection. If this SQL object was created from a DataSource then
+     * this method does nothing.
+     *
+     * @throws SQLException if a database access error occurs
+     */
+    public void rollback() throws SQLException {
+        if (useConnection == null) {
+            log.log(Level.FINEST, "Rollback operation not supported when using datasets unless using withTransaction or cacheConnection - attempt to rollback ignored");
+            return;
+        }
         try {
-            this.useConnection.rollback();
+            useConnection.rollback();
         }
         catch (SQLException e) {
-            log.log(Level.SEVERE, "Caught exception rollbacking connection: " + e, e);
+            log.log(Level.SEVERE, "Caught exception rolling back connection: " + e, e);
+            throw e;
         }
     }
 
@@ -1234,10 +1250,16 @@ public class Sql {
         return useConnection;
     }
 
-
     /**
-     * Allows a closure to be passed in to configure the JDBC statements before they are executed
-     * to do things like set the query size etc.
+     * Allows a closure to be passed in to configure the JDBC statements before they are executed.
+     * It can be used to do things like set the query size etc. When this method is invoked, the supplied
+     * closure is saved. Statements subsequent created from other methods will then be
+     * configured using this closure. The statement being configured is passed into the closure
+     * as its single argument, e.g.:
+     * <pre>
+     * sql.withStatement{ stmt -> stmt.maxRows == 10 }
+     * def firstTenRows = sql.rows("select * from table")
+     * </pre>
      *
      * @param configureStatement the closure
      */
@@ -1249,8 +1271,10 @@ public class Sql {
     //-------------------------------------------------------------------------
 
     /**
+     * Hook to allow derived classes to override sql generation from Gstrings.
+     *
      * @param gstring a GString containing the SQL query with embedded params
-     * @param values the values to embed
+     * @param values  the values to embed
      * @return the SQL version of the given query using ? instead of any
      *         parameter
      */
@@ -1310,7 +1334,8 @@ public class Sql {
     }
 
     /**
-     * replace ?'"? references with NULLish
+     * Hook to allow derived classes to override null handling.
+     * Default behavior is to replace ?'"? references with NULLish
      *
      * @param sql the SQL statement
      * @return the modified SQL String
@@ -1340,7 +1365,9 @@ public class Sql {
     }
 
     /**
-     * Find the first 'where' keyword in the sql.
+     * Hook to allow derived classes to override where clause sniffing.
+     * Default behavior is to find the first 'where' keyword in the sql
+     * doing simple avoidance of the word 'where' within quotes.
      *
      * @param sql the SQL statement
      * @return the index of the found keyword or -1 if not found
@@ -1350,9 +1377,8 @@ public class Sql {
         char[] whereChars = "where".toCharArray();
         int i = 0;
         boolean inString = false; //TODO: Cater for comments?
-        boolean noWhere = true;
         int inWhere = 0;
-        while (i < chars.length && noWhere) {
+        while (i < chars.length) {
             switch (chars[i]) {
                 case '\'':
                     inString = !inString;
@@ -1371,22 +1397,28 @@ public class Sql {
     }
 
     /**
+     * Hook to allow derived classes to override behavior associated with
+     * extracting params from a GString.
+     *
      * @param gstring a GString containing the SQL query with embedded params
      * @return extracts the parameters from the expression as a List
      */
     protected List getParameters(GString gstring) {
-        return new ArrayList(Arrays.asList(gstring.getValues()));
+        return new ArrayList<Object>(Arrays.asList(gstring.getValues()));
     }
 
     /**
-     * Appends the parameters to the given statement.
+     * Hook to allow derived classes to override behavior associated with
+     * setting params for a prepared statement. Default behavior is to
+     * append the parameters to the given statement using <code>setObject</code>.
      *
+     * @param params the parameters to append
+     * @param statement the statement
      * @throws SQLException if a database access error occurs
      */
     protected void setParameters(List params, PreparedStatement statement) throws SQLException {
         int i = 1;
-        for (Iterator iter = params.iterator(); iter.hasNext();) {
-            Object value = iter.next();
+        for (Object value : params) {
             setObject(statement, i++, value);
         }
     }
@@ -1395,6 +1427,9 @@ public class Sql {
      * Strategy method allowing derived classes to handle types differently
      * such as for CLOBs etc.
      *
+     * @param statement the statement of interest
+     * @param i the index of the object of interest
+     * @param value the new object value
      * @throws SQLException if a database access error occurs
      */
     protected void setObject(PreparedStatement statement, int i, Object value)
@@ -1422,15 +1457,25 @@ public class Sql {
         }
     }
 
+    /**
+     * An extension point allowing derived classes to change the behavior of
+     * connection creation. The default behavior is to either use the
+     * supplied connection or obtain it from the supplied datasource.
+     *
+     * @return the connection associated with this Sql
+     * @throws java.sql.SQLException if a SQL error occurs
+     */
     protected Connection createConnection() throws SQLException {
+        if ((cacheStatements || cacheConnection) && useConnection != null) {
+            return useConnection;
+        }
         if (dataSource != null) {
-            //Use a doPrivileged here as many different properties need to be
-            // read, and the policy
-            //shouldn't have to list them all.
-            Connection con = null;
+            // Use a doPrivileged here as many different properties need to be
+            // read, and the policy shouldn't have to list them all.
+            Connection con;
             try {
-                con = (Connection) AccessController.doPrivileged(new PrivilegedExceptionAction() {
-                    public Object run() throws SQLException {
+                con = AccessController.doPrivileged(new PrivilegedExceptionAction<Connection>() {
+                    public Connection run() throws SQLException {
                         return dataSource.getConnection();
                     }
                 });
@@ -1443,52 +1488,66 @@ public class Sql {
                     throw (RuntimeException) e;
                 }
             }
+            if (cacheStatements || cacheConnection) {
+                useConnection = con;
+            }
             return con;
-        } else {
-            return useConnection;
         }
+        return useConnection;
     }
 
+    /**
+     * An extension point allowing derived classes to change the behavior
+     * of resource closing.
+     *
+     * @param connection the connection to close
+     * @param statement the statement to close
+     * @param results the results to close
+     */
     protected void closeResources(Connection connection, Statement statement, ResultSet results) {
         if (results != null) {
             try {
                 results.close();
             }
             catch (SQLException e) {
-                log.log(Level.SEVERE, "Caught exception closing resultSet: " + e, e);
+                log.log(Level.INFO, "Caught exception closing resultSet: " + e, e);
             }
         }
         closeResources(connection, statement);
     }
 
+    /**
+     * An extension point allowing the behavior of resource closing to be
+     * overriden in derived classes.
+     *
+     * @param connection the connection to close
+     * @param statement the statement to close
+     */
     protected void closeResources(Connection connection, Statement statement) {
+        if (cacheStatements) return;
         if (statement != null) {
             try {
                 statement.close();
             }
             catch (SQLException e) {
-                log.log(Level.SEVERE, "Caught exception closing statement: " + e, e);
+                log.log(Level.INFO, "Caught exception closing statement: " + e.getMessage() + " - continuing", e);
             }
         }
+        if (cacheConnection) return;
         if (dataSource != null) {
             try {
                 connection.close();
             }
             catch (SQLException e) {
-                log.log(Level.SEVERE, "Caught exception closing connection: " + e, e);
+                log.log(Level.INFO, "Caught exception closing connection: " + e.getMessage() + " - continuing", e);
             }
         }
     }
 
-    private void warnDeprecated() {
-        if (!warned) {
-            warned = true;
-            log.warning("queryEach() is deprecated, please use eachRow() instead");
-        }
-    }
-
     /**
-     * Provides a hook to be able to configure JDBC statements, such as to configure
+     * Provides a hook for derived classes to be able to configure JDBC statements.
+     * Default behavior is to call a previously saved closure, if any, using the
+     * statement as a parameter.
      *
      * @param statement the statement to configure
      */
@@ -1497,4 +1556,185 @@ public class Sql {
             configureStatement.call(statement);
         }
     }
+
+    /**
+     * Enables statement caching.</br>
+     * if <i>b</i> is true, cache is created and all created prepared statements will be cached.</br>
+     * if <i>b</i> is false, all cached statements will be properly closed.
+     *
+     * @param cacheStatements the new value
+     */
+    public synchronized void setCacheStatements(boolean cacheStatements) {
+        this.cacheStatements = cacheStatements;
+        if (!cacheStatements) {
+            clearStatementCache();
+        }
+    }
+
+    /**
+     * @return boolean    true if cache is enabled
+     */
+    public boolean isCacheStatements() {
+        return cacheStatements;
+    }
+
+    /**
+     * Caches the connection used while the closure is active.
+     * If the closure takes a single argument, it will be called
+     * with the connection, otherwise it will be called with no arguments.
+     *
+     * @param closure the given closure
+     * @throws SQLException if a database error occurs
+     */
+    public synchronized void cacheConnection(Closure closure) throws SQLException {
+        boolean savedCacheConnection = cacheConnection;
+        cacheConnection = true;
+        Connection connection = null;
+        try {
+            connection = createConnection();
+            callClosurePossiblyWithConnection(closure, connection);
+        }
+        finally {
+            cacheConnection = false;
+            closeResources(connection, null);
+            cacheConnection = savedCacheConnection;
+            if (dataSource != null && !cacheConnection) {
+                useConnection = null;
+            }
+        }
+    }
+
+    /**
+     * Performs the closure within a transaction using a cached connection.
+     * If the closure takes a single argument, it will be called
+     * with the connection, otherwise it will be called with no arguments.
+     *
+     * @param closure the given closure
+     * @throws SQLException if a database error occurs
+     */
+    public synchronized void withTransaction(Closure closure) throws SQLException {
+        boolean savedCacheConnection = cacheConnection;
+        cacheConnection = true;
+        Connection connection = null;
+        try {
+            connection = createConnection();
+            connection.setAutoCommit(false);
+            callClosurePossiblyWithConnection(closure, connection);
+            connection.commit();
+        } catch (SQLException e) {
+            handleError(connection, e);
+            throw e;
+        } catch (RuntimeException e) {
+            handleError(connection, e);
+            throw e;
+        } catch (Error e) {
+            handleError(connection, e);
+            throw e;
+        } finally {
+            if (connection != null) connection.setAutoCommit(true);
+            cacheConnection = false;
+            closeResources(connection, null);
+            cacheConnection = savedCacheConnection;
+            if (dataSource != null && !cacheConnection) {
+                useConnection = null;
+            }
+        }
+    }
+
+    private void handleError(Connection connection, Throwable t) throws SQLException {
+        if (connection != null) {
+            log.log(Level.INFO, "Rolling back due to: " + t.getMessage(), t);
+            connection.rollback();
+        }
+    }
+
+    private void callClosurePossiblyWithConnection(Closure closure, Connection connection) {
+        if (closure.getMaximumNumberOfParameters() == 1) {
+            closure.call(connection);
+        } else {
+            closure.call();
+        }
+    }
+
+    /**
+     * Caches every created preparedStatement in closure <i>closure</i></br>
+     * Every cached preparedStatement is closed after closure has been called.
+     * If the closure takes a single argument, it will be called
+     * with the connection, otherwise it will be called with no arguments.
+     *
+     * @param closure the given closure
+     * @throws SQLException if a database error occurs
+     * @see #setCacheStatements(boolean)
+     */
+    public synchronized void cacheStatements(Closure closure) throws SQLException {
+        boolean savedCacheStatements = cacheStatements;
+        cacheStatements = true;
+        Connection connection = null;
+        try {
+            connection = createConnection();
+            callClosurePossiblyWithConnection(closure, connection);
+        }
+        finally {
+            cacheStatements = false;
+            closeResources(connection, null);
+            cacheStatements = savedCacheStatements;
+        }
+    }
+
+    private synchronized void clearStatementCache() {
+        if (!statementCache.isEmpty()) {
+            for (Object o : statementCache.values()) {
+                try {
+                    ((Statement) o).close();
+                } catch (SQLException e) {
+                    log.log(Level.FINEST, "Failed to close statement. Already closed?", e);
+                }
+            }
+            statementCache.clear();
+        }
+    }
+
+    private Statement getStatement(Connection connection, String sql) throws SQLException {
+        Statement stmt;
+        if (cacheStatements) {
+            stmt = statementCache.get(sql);
+            if (stmt == null) {
+                synchronized (statementCache) {
+                    stmt = createStatement(connection);
+                    statementCache.put(sql, stmt);
+                }
+            }
+        } else {
+            stmt = createStatement(connection);
+        }
+        return stmt;
+    }
+
+    private PreparedStatement getPreparedStatement(Connection connection, String sql, int returnGeneratedKeys) throws SQLException {
+        PreparedStatement pStmt;
+        if (cacheStatements) {
+            pStmt = (PreparedStatement) statementCache.get(sql);
+            if (pStmt == null) {
+                synchronized (statementCache) {
+                    pStmt = createPreparedStatement(connection, sql, returnGeneratedKeys);
+                    statementCache.put(sql, pStmt);
+                }
+            }
+        } else {
+            pStmt = createPreparedStatement(connection, sql, returnGeneratedKeys);
+        }
+        return pStmt;
+    }
+
+    private PreparedStatement createPreparedStatement(Connection connection, String sql, int returnGeneratedKeys) throws SQLException {
+        if (returnGeneratedKeys != 0) {
+            return connection.prepareStatement(sql, returnGeneratedKeys);
+        } else {
+            return connection.prepareStatement(sql);
+        }
+    }
+
+    private PreparedStatement getPreparedStatement(Connection connection, String sql) throws SQLException {
+        return getPreparedStatement(connection, sql, 0);
+    }
 }
