@@ -195,9 +195,22 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             type.setName(name);
             if (resolve(type)) return true;
         }
+        if(resolveToInnerEnum (type)) return true;
+        
         type.setName(saved);
         return false;
     }
+    
+    private boolean resolveToInnerEnum (ClassNode type) {
+        // GROOVY-3483: It may be an inner enum defined by this class itself, in which case it does not need to be
+        // explicitly qualified by the currentClass name
+    	String name = type.getName();
+        if(currentClass != type && !name.contains(".") && type.getClass().equals(ClassNode.class)) {
+            type.setName(currentClass.getName() + "$" + name);
+            if (resolve(type)) return true;
+        }
+        return false;
+    }
 
     private void resolveOrFail(ClassNode type, String msg, ASTNode node) {
         if (resolve(type)) return;
@@ -355,7 +368,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
 
     private boolean resolveFromStaticInnerClasses(ClassNode type, boolean testStaticInnerClasses) {
         // a class consisting of a vanilla name can never be
-        // a static inner class, because at last one dot is
+        // a static inner class, because at least one dot is
         // required for this. Example: foo.bar -> foo$bar
         if (type instanceof LowerCaseClass) return false;
 
@@ -666,7 +679,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             if (it instanceof VariableExpression) {
                 VariableExpression ve = (VariableExpression) it;
                 // stop at super and this
-                if (ve == VariableExpression.SUPER_EXPRESSION || ve == VariableExpression.THIS_EXPRESSION) {
+                if (ve.isSuperExpression() || ve.isThisExpression()) {
                     return null;
                 }
                 String varName = ve.getName();
@@ -740,6 +753,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         String propertyNamePart = classPropertyExpression.getPropertyAsString();
         if (propertyNamePart == null || !propertyNamePart.equals("class")) return pe;
 
+        found.setSourcePosition(classPropertyExpression);
         if (stack.isEmpty()) return found;
         stackElement = stack.removeFirst();
         if (!(stackElement.getClass() == PropertyExpression.class)) return pe;
@@ -757,7 +771,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         inPropertyExpression = true;
         isTopLevelProperty = !(objectExpression.getClass() == PropertyExpression.class);
         objectExpression = transform(objectExpression);
-        // we handle the property part as if it where not part of the property
+        // we handle the property part as if it were not part of the property
         inPropertyExpression = false;
         Expression property = transform(pe.getProperty());
         isTopLevelProperty = itlp;
@@ -794,8 +808,6 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     }
 
     protected Expression transformVariableExpression(VariableExpression ve) {
-        if (ve.getName().equals("this")) return VariableExpression.THIS_EXPRESSION;
-        if (ve.getName().equals("super")) return VariableExpression.SUPER_EXPRESSION;
         Variable v = ve.getAccessedVariable();
         if (v instanceof DynamicVariable){
             String name = ve.getName();
@@ -813,6 +825,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                   t = new LowerCaseClass(name);
                 }
                 isClass = resolve(t);
+                if(!isClass) isClass = resolveToInnerEnum(t);
             }
             if (isClass) {
                 // the name is a type so remove it from the scoping
@@ -871,6 +884,12 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             for (int i = 0; i < paras.length; i++) {
                 ClassNode t = paras[i].getType();
                 resolveOrFail(t, ce);
+                if(paras[i].hasInitialExpression()) {
+                	Object initialVal = paras[i].getInitialExpression();
+                	if(initialVal instanceof Expression) {
+                		transform((Expression)initialVal);
+                	}
+                }
             }
         }
         Statement code = ce.getCode();
@@ -904,14 +923,14 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     protected Expression transformDeclarationExpression(DeclarationExpression de) {
         Expression oldLeft = de.getLeftExpression();
         Expression left = transform(oldLeft);
-        if (left != oldLeft) {
+        if (left instanceof ClassExpression) {
             ClassExpression ce = (ClassExpression) left;
             addError("you tried to assign a value to the class " + ce.getType().getName(), oldLeft);
             return de;
         }
         Expression right = transform(de.getRightExpression());
         if (right == de.getRightExpression()) return de;
-        DeclarationExpression newDeclExpr = new DeclarationExpression((VariableExpression) left, de.getOperation(), right);
+        DeclarationExpression newDeclExpr = new DeclarationExpression(left, de.getOperation(), right);
         newDeclExpr.setSourcePosition(de);
         return newDeclExpr;
     }
@@ -930,9 +949,9 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     }
 
     public void visitAnnotations(AnnotatedNode node) {
-        Map annotionMap = node.getAnnotations();
-        if (annotionMap.isEmpty()) return;
-        Iterator it = annotionMap.values().iterator();
+        List annotions = node.getAnnotations();
+        if (annotions.isEmpty()) return;
+        Iterator it = annotions.iterator();
         while (it.hasNext()) {
             AnnotationNode an = (AnnotationNode) it.next();
             //skip builtin properties
@@ -973,6 +992,15 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             for (Iterator iter = importPackages.values().iterator(); iter.hasNext();) {
                 ClassNode type = (ClassNode) iter.next();
                 if (resolve(type, false, false, true)) continue;
+                // May be this type belongs in the same package as the node that is doing the
+                // static import. In that case, the package may not have been explicitly specified.
+                // Try with the node's package too. If still not found, revert to original type name.
+                if(type.getPackageName() == null && node.getPackageName() != null) {
+                	String oldTypeName = type.getName();
+                	type.setName(node.getPackageName() + "." + oldTypeName);
+                	if (resolve(type, false, false, true)) continue;
+                	type.setName(oldTypeName);
+                }
                 addError("unable to resolve class " + type.getName(), type);
             }
             for (Iterator iter = module.getStaticImportAliases().values().iterator(); iter.hasNext();) {
