@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2009 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ package org.codehaus.groovy.control;
 import org.codehaus.groovy.ast.*;
 import org.codehaus.groovy.ast.expr.*;
 import org.codehaus.groovy.ast.stmt.Statement;
+import org.objectweb.asm.Opcodes;
 
 import java.util.*;
 
@@ -31,7 +32,7 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
     private ClassNode currentClass;
     private MethodNode currentMethod;
     private SourceUnit source;
-    private CompilationUnit compilationUnit;
+    private CompilationUnit compilationUnit; // TODO use it or lose it
     private boolean stillResolving;
     private boolean inSpecialConstructorCall;
     private boolean inClosure;
@@ -93,7 +94,10 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
         Variable v = ve.getAccessedVariable();
         if (v != null && v instanceof DynamicVariable) {
             Expression result = findStaticFieldImportFromModule(v.getName());
-            if (result != null) return result;
+            if (result != null) {
+            	result.setSourcePosition(ve);
+            	return result;
+            }
             if (!inPropertyExpression || inSpecialConstructorCall) addStaticVariableError(ve);
         }
         return ve;
@@ -107,10 +111,6 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
         if (object instanceof VariableExpression) {
             VariableExpression ve = (VariableExpression) object;
             isExplicitThisOrSuper = !mce.isImplicitThis() && (ve.getName().equals("this") || ve.getName().equals("super"));
-            if (isExplicitThisOrSuper && currentMethod != null && currentMethod.isStatic()) {
-                addError("Non-static variable '" + ve.getName() + "' cannot be referenced from the static method " + currentMethod.getName() + ".", mce);
-                return null;
-            }
         }
 
         if (mce.isImplicitThis() || isExplicitThisOrSuper) {
@@ -124,7 +124,14 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
                 Object value = ce.getValue();
                 if (value instanceof String) {
                     String methodName = (String) value;
-                    if (inSpecialConstructorCall || currentClass.hasPossibleStaticMethod(methodName, args)) {
+                    boolean lookForPossibleStaticMethod = true;
+                    if(this.currentMethod != null && !this.currentMethod.isStatic()) {
+                        if(currentClass.hasPossibleMethod(methodName, args)) {
+                            lookForPossibleStaticMethod = false;
+                        }
+                    }
+                    if (inSpecialConstructorCall ||
+                            (lookForPossibleStaticMethod && currentClass.hasPossibleStaticMethod(methodName, args))) {
                     	StaticMethodCallExpression smce = new StaticMethodCallExpression(currentClass, methodName, args);
                     	smce.setSourcePosition(mce);
                     	return smce;
@@ -170,9 +177,10 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
         if (objectExpression instanceof ClassExpression && currentMethod != null && currentMethod.isStatic()) {
             ClassExpression ce = (ClassExpression) objectExpression;
             if (ce.getType().getName().equals(currentClass.getName())) {
-                FieldNode field = currentClass.getField(pe.getPropertyAsString());
+                FieldNode field = currentClass.getDeclaredField(pe.getPropertyAsString());
                 if (field != null && field.isStatic()) {
                     Expression expression = new FieldExpression(field);
+                    expression.setSourcePosition(pe);
                     return expression;
                 }
             }
@@ -219,12 +227,15 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
         // propertiesExpressions will handle the error a bit different
         if (!inSpecialConstructorCall && (inClosure || !ve.isInStaticContext())) return;
         if (stillResolving) return;
-        if (ve == VariableExpression.THIS_EXPRESSION || ve == VariableExpression.SUPER_EXPRESSION) return;
+        if (ve.isThisExpression() || ve.isSuperExpression()) return;
         Variable v = ve.getAccessedVariable();
         if (v != null && !(v instanceof DynamicVariable) && v.isInStaticContext()) return;
-        addError("The name " + ve.getName() + " doesn't refer to a declared variable or class. The static" +
-                " scope requires that you declare variables before using them. If the variable should have" +
-                " been a class check the spelling.", ve);
+        addError("Apparent variable '" + ve.getName() + "' was found in a static scope but doesn't refer" +
+                " to a local variable, static field or class. Possible causes:\n" +
+                "You attemped to reference a variable in the binding or an instance variable from a static context.\n" +
+                "You mispelled a classname or statically imported field. Please check the spelling.\n" +
+                "You attempted to use a method '" + ve.getName() +
+                "' but left out brackets in a place not allowed by the grammar.", ve);
     }
 
     private Expression findStaticFieldImportFromModule(String name) {
@@ -240,9 +251,8 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
             if (expression != null) return expression;
         }
         Map importedClasses = module.getStaticImportClasses();
-        Iterator it = importedClasses.keySet().iterator();
-        while (it.hasNext()) {
-            String className = (String) it.next();
+        for (Object o : importedClasses.keySet()) {
+            String className = (String) o;
             ClassNode node = (ClassNode) importedClasses.get(className);
             Expression expression = findStaticField(node, name);
             if (expression != null) return expression;
@@ -251,15 +261,14 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
     }
 
     private Expression findStaticField(ClassNode staticImportType, String fieldName) {
-        if (!staticImportType.isResolved() && !staticImportType.isPrimaryClassNode()) {
-            stillResolving = true;
-        }
         if (staticImportType.isPrimaryClassNode() || staticImportType.isResolved()) {
             staticImportType.getFields(); // force init
             FieldNode field = staticImportType.getField(fieldName);
             if (field != null && field.isStatic()) {
                 return new PropertyExpression(new ClassExpression(staticImportType), fieldName);
             }
+        } else {
+            stillResolving = true;
         }
         return null;
     }
@@ -281,10 +290,14 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
             if (expression != null) return expression;
         }
         Map importPackages = module.getStaticImportClasses();
-        Iterator it = importPackages.keySet().iterator();
-        while (it.hasNext()) {
-            String className = (String) it.next();
-            ClassNode starImportType = (ClassNode) importPackages.get(className);
+        for (Object o : importPackages.keySet()) {
+            String className = (String) o;
+            ClassNode starImportType = null;
+            if(isEnum(currentClass) && importPackages.containsKey(currentClass.getName())) {
+            	starImportType = (ClassNode) importPackages.get(currentClass.getName());
+            } else {
+                starImportType = (ClassNode) importPackages.get(className);
+            }
             Expression expression = findStaticMethod(starImportType, name, args);
             if (expression != null) return expression;
         }
@@ -303,4 +316,8 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
     protected SourceUnit getSourceUnit() {
         return source;
     }
+
+    private boolean isEnum(ClassNode node) {
+        return (node.getModifiers() & Opcodes.ACC_ENUM) != 0;
+     }
 }
