@@ -95,7 +95,7 @@ public class ParameterTypes
       if (parameterTypes != null) {
           npt = new Class [parameterTypes.length];
           for (int i = 0; i != parameterTypes.length; ++i) {
-              npt[i] = parameterTypes[i].getCachedClass();
+              npt[i] = parameterTypes[i].getTheClass();
           }
       }
       else
@@ -121,7 +121,7 @@ public class ParameterTypes
         Object last = arguments[arguments.length - 1];
         if (last == null) return true;
         Class clazz = last.getClass();
-        return !clazz.equals(parameterTypes[lenMinus1].getCachedClass());
+        return !clazz.equals(parameterTypes[lenMinus1].getTheClass());
 
     }
 
@@ -141,7 +141,7 @@ public class ParameterTypes
         return argumentArray;
     }
 
-    private Object[] correctArguments(Object[] argumentArray) {
+    public Object[] correctArguments(Object[] argumentArray) {
         // correct argumentArray's length
         if (argumentArray == null) {
             return MetaClassHelper.EMPTY_ARRAY;
@@ -150,7 +150,7 @@ public class ParameterTypes
         final CachedClass[] pt = parameterTypes;
         if (pt.length == 1 && argumentArray.length == 0) {
             if (isVargsMethod)
-                return new Object[]{Array.newInstance(pt[0].getCachedClass().getComponentType(), 0)};
+                return new Object[]{Array.newInstance(pt[0].getTheClass().getComponentType(), 0)};
             else
                 return MetaClassHelper.ARRAY_WITH_NULL;
         }
@@ -171,7 +171,7 @@ public class ParameterTypes
      * @param paramTypes    the types of the paramters the method takes
      */
     private static Object[] fitToVargs(Object[] argumentArray, CachedClass[] paramTypes) {
-        Class vargsClass = ReflectionCache.autoboxType(paramTypes[paramTypes.length - 1].getCachedClass().getComponentType());
+        Class vargsClass = ReflectionCache.autoboxType(paramTypes[paramTypes.length - 1].getTheClass().getComponentType());
 
         if (argumentArray.length == paramTypes.length - 1) {
             // the vargs argument is missing, so fill it with an empty array
@@ -220,26 +220,64 @@ public class ParameterTypes
         CachedClass[] pt = getParameterTypes();
         final int paramMinus1 = pt.length-1;
 
-        if ( size >= paramMinus1 && isVargsMethod)
+        if (isVargsMethod && size >= paramMinus1)
             return isValidVarargsMethod(arguments, size, pt, paramMinus1);
         else
             if (pt.length == size)
-                return isValidExactMethod(arguments, size, pt);
+                return isValidExactMethod(arguments, pt);
             else
                 if (pt.length == 1 && size == 0)
                     return true;
         return false;
     }
 
-    private boolean isValidExactMethod(Class[] arguments, int size, CachedClass[] pt) {
+    private boolean isValidExactMethod(Class[] arguments, CachedClass[] pt) {
         // lets check the parameter types match
+        int size = pt.length;
         for (int i = 0; i < size; i++) {
-            if (pt[i].isAssignableFrom(arguments[i])) continue;
-            return false;
+            if (!pt[i].isAssignableFrom(arguments[i])) {
+                return false;
+            }
         }
         return true;
     }
 
+    public boolean isValidExactMethod(Object [] args) {
+        // lets check the parameter types match
+        getParametersTypes0();
+        int size = args.length;
+        if (size != parameterTypes.length)
+          return false;
+        
+        for (int i = 0; i < size; i++) {
+            if (args[i] != null && !parameterTypes[i].isAssignableFrom(args[i].getClass())) {
+                return false;
+            }
+        }
+        return true;
+    }
+
+    public boolean isValidExactMethod(Class [] args) {
+        // lets check the parameter types match
+        getParametersTypes0();
+        int size = args.length;
+        if (size != parameterTypes.length)
+          return false;
+
+        for (int i = 0; i < size; i++) {
+            if (args[i] != null && !parameterTypes[i].isAssignableFrom(args[i])) {
+                return false;
+            }
+        }
+        return true;
+    }
+
+    private final static boolean testComponentAssignable(Class toTestAgainst, Class toTest) {
+        Class component = toTest.getComponentType();
+        if (component==null) return false;
+        return MetaClassHelper.isAssignableFrom(toTestAgainst, component);
+    }
+
     private boolean isValidVarargsMethod(Class[] arguments, int size, CachedClass[] pt, int paramMinus1) {
         // first check normal number of parameters
         for (int i = 0; i < paramMinus1; i++) {
@@ -249,7 +287,7 @@ public class ParameterTypes
 
         // check direct match
         CachedClass varg = pt[paramMinus1];
-        Class clazz = varg.getCachedClass().getComponentType();
+        Class clazz = varg.getTheClass().getComponentType();
         if ( size==pt.length &&
              (varg.isAssignableFrom(arguments[paramMinus1]) ||
               testComponentAssignable(clazz, arguments[paramMinus1])))
@@ -265,12 +303,6 @@ public class ParameterTypes
         return true;
     }
 
-    private final static boolean testComponentAssignable(Class toTestAgainst, Class toTest) {
-        Class component = toTest.getComponentType();
-        if (component==null) return false;
-        return MetaClassHelper.isAssignableFrom(toTestAgainst, component);
-    }
-
     public boolean isValidMethod(Object[] arguments) {
         if (arguments == null) return true;
 
@@ -290,10 +322,10 @@ public class ParameterTypes
             
             // check direct match
             CachedClass varg = paramTypes[paramMinus1];
-            Class clazz = varg.getCachedClass().getComponentType();
+            Class clazz = varg.getTheClass().getComponentType();
             if ( size==paramTypes.length && 
                  (varg.isAssignableFrom(getArgClass(arguments[paramMinus1])) ||
-                  testComponentAssignable(clazz,getArgClass(arguments[paramMinus1]))))
+                  testComponentAssignable(clazz, getArgClass(arguments[paramMinus1])))) 
             {
                 return true;
             }
