@@ -15,17 +15,23 @@
  */
 package org.codehaus.groovy.runtime;
 
-import groovy.lang.IntRange;
 import groovy.lang.EmptyRange;
+import groovy.lang.IntRange;
 import org.codehaus.groovy.runtime.typehandling.DefaultTypeTransformation;
 
+import java.io.Closeable;
+import java.io.IOException;
+import java.lang.reflect.Constructor;
 import java.util.*;
+import java.util.logging.Logger;
 
 /**
  * Support methods for DefaultGroovyMethods and PluginDefaultMethods.
  */
 public class DefaultGroovyMethodsSupport {
 
+    private static final Logger LOG = Logger.getLogger(DefaultGroovyMethodsSupport.class.getName());
+
     // helper method for getAt and putAt
     protected static RangeInfo subListBorders(int size, IntRange range) {
         int from = normaliseIndex(DefaultTypeTransformation.intUnbox(range.getFrom()), size);
@@ -65,6 +71,36 @@ public class DefaultGroovyMethodsSupport {
         return i;
     }
 
+    /**
+     * Close the Closeable. Logging a warning if any problems occur.
+     *
+     * @param c the thing to close
+     */
+    public static void closeWithWarning(Closeable c) {
+        if (c != null) {
+            try {
+                c.close();
+            } catch (IOException e) {
+                LOG.warning("Caught exception during close(): " + e);
+            }
+        }
+    }
+
+    /**
+     * Close the Closeable. Ignore any problems that might occur.
+     *
+     * @param c the thing to close
+     */
+    public static void closeQuietly(Closeable c) {
+        if (c != null) {
+            try {
+                c.close();
+            } catch (IOException e) {
+                /* ignore */
+            }
+        }
+    }
+
     protected static class RangeInfo {
         public int from, to;
         public boolean reverse;
@@ -76,12 +112,27 @@ public class DefaultGroovyMethodsSupport {
         }
     }
 
-    protected static Collection cloneSimilarCollection(Collection left, int newCapacity) {
-        Collection answer = createSimilarCollection(left, newCapacity);
-        answer.addAll(left);
+    protected static Collection cloneSimilarCollection(Collection orig, int newCapacity) {
+        Collection answer = (Collection) cloneObject(orig);
+        if (answer != null) return answer;
+
+        // fall back to creation
+        answer = createSimilarCollection(orig, newCapacity);
+        answer.addAll(orig);
         return answer;
     }
 
+    private static Object cloneObject(Object orig) {
+        if (orig instanceof Cloneable) {
+            try {
+                return InvokerHelper.invokeMethod(orig, "clone", new Object[0]);
+            } catch (Exception ex) {
+                // ignore
+            }
+        }
+        return null;
+    }
+
     protected static Collection createSimilarOrDefaultCollection(Object object) {
         if (object instanceof Collection) {
             return createSimilarCollection((Collection) object);
@@ -93,54 +144,73 @@ public class DefaultGroovyMethodsSupport {
         return createSimilarCollection(collection, collection.size());
     }
 
-    protected static Collection createSimilarCollection(Collection collection, int newCapacity) {
-        if (collection instanceof Set) {
-            return createSimilarSet((Set) collection);
+    protected static Collection createSimilarCollection(Collection orig, int newCapacity) {
+        if (orig instanceof Set) {
+            return createSimilarSet((Set) orig);
         }
-        if (collection instanceof List) {
-            return createSimilarList((List) collection, newCapacity);
+        if (orig instanceof List) {
+            return createSimilarList((List) orig, newCapacity);
+        }
+        if (orig instanceof Queue) {
+            return new LinkedList();
         }
         return new ArrayList(newCapacity);
     }
 
-    protected static List createSimilarList(List left, int newCapacity) {
-        List answer;
-        if (left instanceof LinkedList) {
-            answer = new LinkedList();
-        } else if (left instanceof Vector) {
-            answer = new Vector();
-        } else {
-            answer = new ArrayList(newCapacity);
+    protected static List createSimilarList(List orig, int newCapacity) {
+        if (orig instanceof LinkedList) {
+            return new LinkedList();
         }
-        return answer;
+        if (orig instanceof Stack) {
+            return new Stack();
+        }
+        if (orig instanceof Vector) {
+            return new Vector();
+        }
+        return new ArrayList(newCapacity);
     }
 
-    protected static Map cloneSimilarMap(Map left) {
-        Map map;
-        if (left instanceof TreeMap)
-            map = new TreeMap(left);
-        else if (left instanceof LinkedHashMap)
-            map = new LinkedHashMap(left);
-        else if (left instanceof Properties) {
-            map = new Properties();
-            map.putAll(left);
-        } else if (left instanceof Hashtable)
-            map = new Hashtable(left);
-        else
-            map = new HashMap(left);
-        return map;
+    protected static Set createSimilarSet(Set orig) {
+        if (orig instanceof SortedSet) {
+            return new TreeSet();
+        }
+        if (orig instanceof LinkedHashSet) {
+            return new LinkedHashSet();
+        }
+        return new HashSet();
     }
 
-    protected static Set createSimilarSet(Set self) {
-        final Set ansSet;
-        if (self instanceof SortedSet) {
-            ansSet = new TreeSet();
-        } else if (self instanceof LinkedHashSet) {
-            ansSet = new LinkedHashSet();
-        } else {
-            ansSet = new HashSet();
+    protected static Map createSimilarMap(Map orig) {
+        if (orig instanceof SortedMap) {
+            return new TreeMap();
         }
-        return ansSet;
+        if (orig instanceof Properties) {
+            return new Properties();
+        }
+        if (orig instanceof Hashtable) {
+            return new Hashtable();
+        }
+        return new LinkedHashMap();
+    }
+
+    protected static Map cloneSimilarMap(Map orig) {
+        Map answer = (Map) cloneObject(orig);
+        if (answer != null) return answer;
+
+        // fall back to some defaults
+        if (orig instanceof TreeMap)
+            return new TreeMap(orig);
+
+        if (orig instanceof Properties) {
+            Map map = new Properties();
+            map.putAll(orig);
+            return map;
+        }
+
+        if (orig instanceof Hashtable)
+            return new Hashtable(orig);
+
+        return new LinkedHashMap(orig);
     }
 
     /**
@@ -151,8 +221,8 @@ public class DefaultGroovyMethodsSupport {
      */
     protected static boolean sameType(Collection[] cols) {
         List all = new LinkedList();
-        for (int i = 0; i < cols.length; i++) {
-            all.addAll(cols[i]);
+        for (Collection col : cols) {
+            all.addAll(col);
         }
         if (all.size() == 0)
             return true;
@@ -164,13 +234,15 @@ public class DefaultGroovyMethodsSupport {
         Class baseClass;
         if (first instanceof Number) {
             baseClass = Number.class;
+        } else if (first == null) {
+            baseClass = NullObject.class;
         } else {
             baseClass = first.getClass();
         }
 
-        for (int i = 0; i < cols.length; i++) {
-            for (Iterator iter = cols[i].iterator(); iter.hasNext();) {
-                if (!baseClass.isInstance(iter.next())) {
+        for (Collection col : cols) {
+            for (Object o : col) {
+                if (!baseClass.isInstance(o)) {
                     return false;
                 }
             }
