@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2008 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,9 +16,10 @@
 package org.codehaus.groovy.runtime;
 
 import groovy.lang.*;
-import groovy.xml.dom.DOMUtil;
+import groovy.xml.XmlUtil;
+import org.codehaus.groovy.runtime.metaclass.MetaClassRegistryImpl;
+import org.codehaus.groovy.runtime.metaclass.MissingMethodExecutionFailed;
 import org.codehaus.groovy.runtime.typehandling.DefaultTypeTransformation;
-import org.codehaus.groovy.runtime.typehandling.IntegerCache;
 import org.codehaus.groovy.runtime.wrappers.PojoWrapper;
 import org.w3c.dom.Element;
 
@@ -34,7 +35,7 @@ import java.util.regex.Pattern;
  * A static helper class to make bytecode generation easier and act as a facade over the Invoker
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 10259 $
+ * @version $Revision: 16276 $
  */
 public class InvokerHelper {
     private   static final Object[] EMPTY_MAIN_ARGS = new Object[]{new String[0]};
@@ -43,28 +44,8 @@ public class InvokerHelper {
     protected static final Object[] EMPTY_ARGUMENTS = EMPTY_ARGS;
     protected static final Class[]  EMPTY_TYPES = {};
 
-    static final MetaClassRegistry metaRegistry = GroovySystem.getMetaClassRegistry();
-    /**
-     * @deprecated
-     */
-    private static final Invoker SINGLETON = new Invoker();
+    public static final MetaClassRegistry metaRegistry = GroovySystem.getMetaClassRegistry();
 
-    /**
-     * This method should no longer be used.
-     * @deprecated
-     * @return an Invoker
-     */
-    public static Invoker getInstance() {
-        return SINGLETON;
-    }
-
-    /**
-     * @deprecated
-     */
-    public static Object invokeNoArgumentsMethod(Object object, String methodName) {
-        return getInstance().invokeMethod(object, methodName, EMPTY_ARGS);
-    }    
-    
     public static void removeClass(Class clazz) {
         metaRegistry.removeMetaClass(clazz);
         Introspector.flushFromCaches(clazz);
@@ -136,46 +117,47 @@ public class InvokerHelper {
     }
 
     public static Object getAttribute(Object object, String attribute) {
-//        if (object == null) {
-//            throw new NullPointerException("Cannot get attribute: " + attribute + " on null object");
-//        }
+        if (object == null) {
+            object = NullObject.getNullObject();
+        }
 
         if (object instanceof Class) {
             return metaRegistry.getMetaClass((Class) object).getAttribute(object, attribute);
-        }
-        if (object instanceof GroovyObject) {
+        } else if (object instanceof GroovyObject) {
             return ((GroovyObject) object).getMetaClass().getAttribute(object, attribute);
+        } else {
+            return metaRegistry.getMetaClass(object.getClass()).getAttribute(object, attribute);
         }
-        return metaRegistry.getMetaClass(object.getClass()).getAttribute(object, attribute);
     }
 
     public static void setAttribute(Object object, String attribute, Object newValue) {
-//        if (object == null) {
-//            throw new GroovyRuntimeException("Cannot set attribute on null object");
-//        }
+        if (object == null) {
+            object = NullObject.getNullObject();
+        }
 
         if (object instanceof Class) {
             metaRegistry.getMetaClass((Class) object).setAttribute(object, attribute, newValue);
-        }
-        if (object instanceof GroovyObject) {
+        } else if (object instanceof GroovyObject) {
             ((GroovyObject) object).getMetaClass().setAttribute(object, attribute, newValue);
+        } else {
+            metaRegistry.getMetaClass(object.getClass()).setAttribute(object, attribute, newValue);
         }
-        metaRegistry.getMetaClass(object.getClass()).setAttribute(object, attribute, newValue);
     }
 
     public static Object getProperty(Object object, String property) {
-//        if (object == null) {
-//            throw new NullPointerException("Cannot get property: " + property + " on null object");
-//        }
+        if (object == null) {
+            object = NullObject.getNullObject();
+        }
+        
         if (object instanceof GroovyObject) {
             GroovyObject pogo = (GroovyObject) object;
             return pogo.getProperty(property);
-        }
-        if (object instanceof Class) {
+        } else if (object instanceof Class) {
             Class c = (Class) object;
             return metaRegistry.getMetaClass(c).getProperty(object, property);
+        } else {
+            return ((MetaClassRegistryImpl)metaRegistry).getMetaClass(object).getProperty(object, property);
         }
-        return metaRegistry.getMetaClass(object.getClass()).getProperty(object, property);
     }
 
     public static Object getPropertySafe(Object object, String property) {
@@ -186,18 +168,17 @@ public class InvokerHelper {
     }
 
     public static void setProperty(Object object, String property, Object newValue) {
-//        if (object == null) {
-//            throw new GroovyRuntimeException("Cannot set property on null object");
-//        }
+        if (object == null) {
+            object = NullObject.getNullObject();
+        }
+
         if (object instanceof GroovyObject) {
             GroovyObject pogo = (GroovyObject) object;
             pogo.setProperty(property, newValue);
-        }
-        else {
-            if (object instanceof Class)
-                metaRegistry.getMetaClass((Class) object).setProperty((Class) object, property, newValue);
-            else
-                metaRegistry.getMetaClass(object.getClass()).setProperty(object, property, newValue);
+        } else if (object instanceof Class) {
+            metaRegistry.getMetaClass((Class) object).setProperty((Class) object, property, newValue);
+        } else {
+            ((MetaClassRegistryImpl) GroovySystem.getMetaClassRegistry()).getMetaClass(object).setProperty(object, property, newValue);
         }
     }
 
@@ -246,7 +227,7 @@ public class InvokerHelper {
     public static Object unaryMinus(Object value) {
         if (value instanceof Integer) {
             Integer number = (Integer) value;
-            return IntegerCache.integerValue(-number.intValue());
+            return Integer.valueOf(-number.intValue());
         }
         if (value instanceof Long) {
             Long number = (Long) value;
@@ -423,7 +404,12 @@ public class InvokerHelper {
                     // though the bindings will be ignored
                     script = new Script() {
                         public Object run() {
-                            object.invokeMethod("main", EMPTY_MAIN_ARGS);
+                        	Object args = getBinding().getVariables().get("args");
+                        	Object argsToPass = EMPTY_MAIN_ARGS;
+                        	if(args != null && args instanceof String[]) {
+                        		argsToPass = args;
+                        	}
+                            object.invokeMethod("main", argsToPass);
                             return null;
                         }
                     };
@@ -513,6 +499,9 @@ public class InvokerHelper {
             return (String) nullObject.getMetaClass().invokeMethod(nullObject, "toString", EMPTY_ARGS);
         }
         if (arguments.getClass().isArray()) {
+            if (arguments instanceof char[]) {
+                return new String((char[]) arguments);
+            }
             return format(DefaultTypeTransformation.asCollection(arguments), verbose);
         }
         if (arguments instanceof Range) {
@@ -523,48 +512,14 @@ public class InvokerHelper {
                 return range.toString();
             }
         }
-        if (arguments instanceof List) {
-            List list = (List) arguments;
-            StringBuffer buffer = new StringBuffer("[");
-            boolean first = true;
-            for (Iterator iter = list.iterator(); iter.hasNext();) {
-                if (first) {
-                    first = false;
-                } else {
-                    buffer.append(", ");
-                }
-                buffer.append(format(iter.next(), verbose));
-            }
-            buffer.append("]");
-            return buffer.toString();
+        if (arguments instanceof Collection) {
+            return formatList((Collection) arguments, verbose);
         }
         if (arguments instanceof Map) {
-            Map map = (Map) arguments;
-            if (map.isEmpty()) {
-                return "[:]";
-            }
-            StringBuffer buffer = new StringBuffer("[");
-            boolean first = true;
-            for (Iterator iter = map.entrySet().iterator(); iter.hasNext();) {
-                if (first) {
-                    first = false;
-                } else {
-                    buffer.append(", ");
-                }
-                Map.Entry entry = (Map.Entry) iter.next();
-                buffer.append(format(entry.getKey(), verbose));
-                buffer.append(":");
-                if (entry.getValue() == map) {
-                    buffer.append("this Map_");
-                } else {
-                    buffer.append(format(entry.getValue(), verbose));
-                }
-            }
-            buffer.append("]");
-            return buffer.toString();
+            return formatMap((Map) arguments, verbose);
         }
         if (arguments instanceof Element) {
-            return DOMUtil.serialize((Element) arguments);
+            return XmlUtil.serialize((Element) arguments);
         }
         if (arguments instanceof String) {
             if (verbose) {
@@ -572,19 +527,63 @@ public class InvokerHelper {
                 arg = arg.replaceAll("\\r", "\\\\r");      // carriage return
                 arg = arg.replaceAll("\\t", "\\\\t");      // tab
                 arg = arg.replaceAll("\\f", "\\\\f");      // form feed
-                arg = arg.replaceAll("\\\"", "\\\\\"");    // double quotation amrk
-                arg = arg.replaceAll("\\\\", "\\\\");      // back slash
+                arg = arg.replaceAll("\\\"", "\\\\\"");    // double quotation mark
+                arg = arg.replaceAll("\\\\", "\\\\");      // backslash
                 return "\"" + arg + "\"";
             } else {
                 return (String) arguments;
             }
         }
+        // TODO: For GROOVY-2599 do we need something like below but it breaks other things
+//        return (String) invokeMethod(arguments, "toString", EMPTY_ARGS);
         return arguments.toString();
     }
 
+    private static String formatMap(Map map, boolean verbose) {
+        if (map.isEmpty()) {
+            return "[:]";
+        }
+        StringBuffer buffer = new StringBuffer("[");
+        boolean first = true;
+        for (Iterator iter = map.entrySet().iterator(); iter.hasNext();) {
+            if (first) {
+                first = false;
+            } else {
+                buffer.append(", ");
+            }
+            Map.Entry entry = (Map.Entry) iter.next();
+            buffer.append(format(entry.getKey(), verbose));
+            buffer.append(":");
+            if (entry.getValue() == map) {
+                buffer.append("this Map_");
+            } else {
+                buffer.append(format(entry.getValue(), verbose));
+            }
+        }
+        buffer.append("]");
+        return buffer.toString();
+    }
+
+    private static String formatList(Collection list, boolean verbose) {
+        StringBuffer buffer = new StringBuffer("[");
+        boolean first = true;
+        for (Iterator iter = list.iterator(); iter.hasNext();) {
+            if (first) {
+                first = false;
+            } else {
+                buffer.append(", ");
+            }
+            buffer.append(format(iter.next(), verbose));
+        }
+        buffer.append("]");
+        return buffer.toString();
+    }
 
     /**
-     * A helper method to format the arguments types as a comma-separated list
+     * A helper method to format the arguments types as a comma-separated list.
+     *
+     * @param arguments the type to process
+     * @return the string representation of the type
      */
     public static String toTypeString(Object[] arguments) {
         if (arguments == null) {
@@ -602,75 +601,43 @@ public class InvokerHelper {
 
     /**
      * A helper method to return the string representation of a map with bracket boundaries "[" and "]".
+     *
+     * @param arg the map to process
+     * @return the string representation of the map
      */
     public static String toMapString(Map arg) {
-        return format(arg, true);
-        /*if (arg == null) {
-            return "null";
-        }
-        if (arg.isEmpty()) {
-            return "[:]";
-        }
-        String sbdry = "[";
-        String ebdry = "]";
-        StringBuffer buffer = new StringBuffer(sbdry);
-        boolean first = true;
-        for (Iterator iter = arg.entrySet().iterator(); iter.hasNext();) {
-            if (first)
-                first = false;
-            else
-                buffer.append(", ");
-            Map.Entry entry = (Map.Entry) iter.next();
-            buffer.append(format(entry.getKey(), true));
-            buffer.append(":");
-            buffer.append(format(entry.getValue(), true));
-        }
-        buffer.append(ebdry);
-        return buffer.toString();*/
+        return formatMap(arg, false);
     }
 
     /**
      * A helper method to return the string representation of a list with bracket boundaries "[" and "]".
+     *
+     * @param arg the collection to process
+     * @return the string representation of the collection
      */
     public static String toListString(Collection arg) {
-        if (arg == null) {
-            return "null";
-        }
-        if (arg.isEmpty()) {
-            return "[]";
-        }
-        String sbdry = "[";
-        String ebdry = "]";
-        StringBuffer buffer = new StringBuffer(sbdry);
-        boolean first = true;
-        for (Iterator iter = arg.iterator(); iter.hasNext();) {
-            if (first)
-                first = false;
-            else
-                buffer.append(", ");
-            Object elem = iter.next();
-            buffer.append(format(elem, true));
-        }
-        buffer.append(ebdry);
-        return buffer.toString();
+        return formatList(arg, false);
     }
 
     /**
-     * A helper method to return the string representation of an arrray of objects
+     * A helper method to return the string representation of an array of objects
      * with brace boundaries "{" and "}".
+     *
+     * @param arguments the array to process
+     * @return the string representation of the array
      */
     public static String toArrayString(Object[] arguments) {
         if (arguments == null) {
             return "null";
         }
-        String sbdry = "{";
-        String ebdry = "}";
+        String sbdry = "[";
+        String ebdry = "]";
         StringBuffer argBuf = new StringBuffer(sbdry);
         for (int i = 0; i < arguments.length; i++) {
             if (i > 0) {
                 argBuf.append(", ");
             }
-            argBuf.append(format(arguments[i], true));
+            argBuf.append(format(arguments[i], false));
         }
         argBuf.append(ebdry);
         return argBuf.toString();
@@ -691,7 +658,7 @@ public class InvokerHelper {
     public static Object bitwiseNegate(Object value) {
         if (value instanceof Integer) {
             Integer number = (Integer) value;
-            return new Integer(~number.intValue());
+            return Integer.valueOf(~number.intValue());
         }
         if (value instanceof Long) {
             Long number = (Long) value;
@@ -725,7 +692,10 @@ public class InvokerHelper {
     }
 
     public static MetaClass getMetaClass(Object object) {
-        return metaRegistry.getMetaClass(object.getClass());
+        if (object instanceof GroovyObject)
+            return ((GroovyObject) object).getMetaClass();
+        else
+            return ((MetaClassRegistryImpl) GroovySystem.getMetaClassRegistry()).getMetaClass(object);
     }
 
     public static MetaClass getMetaClass(Class cls) {
@@ -747,25 +717,22 @@ public class InvokerHelper {
             MetaClass metaClass = metaRegistry.getMetaClass(theClass);
             return metaClass.invokeStaticMethod(object, methodName, asArray(arguments));
         }
-        else { // it's an instance
-            // if it's not an object implementing GroovyObject (thus not builder, nor a closure)
-            if (!(object instanceof GroovyObject)) {
-                return invokePojoMethod(object, methodName, arguments);
-            }
-            // it's an object implementing GroovyObject
-            else {
-                return invokePogoMethod(object, methodName, arguments);
-            }
+
+        // it's an instance; check if it's a Java one
+        if (!(object instanceof GroovyObject)) {
+            return invokePojoMethod(object, methodName, arguments);
         }
+
+        // a groovy instance (including builder, closure, ...)
+        return invokePogoMethod(object, methodName, arguments);
     }
 
-    private static Object invokePojoMethod(Object object, String methodName, Object arguments) {
-        Class theClass = object.getClass();
-        MetaClass metaClass = metaRegistry.getMetaClass(theClass);
+    static Object invokePojoMethod(Object object, String methodName, Object arguments) {
+        MetaClass metaClass = InvokerHelper.getMetaClass(object);
         return metaClass.invokeMethod(object, methodName, asArray(arguments));
     }
 
-    private static Object invokePogoMethod(Object object, String methodName, Object arguments) {
+    static Object invokePogoMethod(Object object, String methodName, Object arguments) {
         GroovyObject groovy = (GroovyObject) object;
         boolean intercepting = groovy instanceof GroovyInterceptable;
         try {
@@ -776,11 +743,14 @@ public class InvokerHelper {
             //else try a statically typed method or a GDK method
             return groovy.getMetaClass().invokeMethod(object, methodName, asArray(arguments));
         } catch (MissingMethodException e) {
-            if (!intercepting && e.getMethod().equals(methodName) && object.getClass() == e.getType()) {
+            if (e instanceof MissingMethodExecutionFailed) {
+                throw (MissingMethodException) e.getCause();
+            } else if (!intercepting && e.getMethod().equals(methodName) && object.getClass() == e.getType()) {
                 // in case there's nothing else, invoke the object's own invokeMethod()
                 return groovy.invokeMethod(methodName, asUnwrappedArray(arguments));
+            } else {
+                throw e;
             }
-            throw e;
         }
     }
 
