@@ -20,10 +20,15 @@ import groovy.lang.Closure;
 import groovy.lang.GString;
 import groovy.lang.GroovyRuntimeException;
 import groovy.lang.MetaMethod;
-import org.codehaus.groovy.reflection.*;
+import org.codehaus.groovy.reflection.CachedClass;
+import org.codehaus.groovy.util.FastArray;
+import org.codehaus.groovy.reflection.ParameterTypes;
+import org.codehaus.groovy.reflection.ReflectionCache;
 import org.codehaus.groovy.runtime.wrappers.Wrapper;
 
-import java.lang.reflect.*;
+import java.lang.reflect.Array;
+import java.lang.reflect.Constructor;
+import java.lang.reflect.Modifier;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.Iterator;
@@ -52,6 +57,7 @@ public class MetaClassHelper {
      * 44-48: vargs penalty
      */
     
+    public static final Class[] EMPTY_CLASS_ARRAY = new Class[0];
 
     public static boolean accessibleToConstructor(final Class at, final Constructor constructor) {
         boolean accessible = false;
@@ -128,7 +134,7 @@ public class MetaClassHelper {
             int[] array = (int[]) parameters;
             ret = new Object[array.length];
             for (int i = 0; i < array.length; i++) {
-                ret[i] = new Integer(array[i]);
+                ret[i] = Integer.valueOf(array[i]);
             }
         } else if (componentType == short.class) {
             short[] array = (short[]) parameters;
@@ -212,7 +218,7 @@ public class MetaClassHelper {
             /*Integer*/{14, 15, 12, 13, 1, 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,},
             /*long*/{14, 15, 12, 13, 10, 11, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,},
             /*Long*/{14, 15, 12, 13, 10, 11, 1, 0, 2, 3, 4, 5, 6, 7, 8, 9,},
-            /*BigInteger*/{14, 15, 12, 13, 10, 11, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7,},
+            /*BigInteger*/{9, 10, 7, 8, 5, 6, 3, 4, 0, 14, 15, 12, 13, 11, 1, 2,},
             /*float*/{14, 15, 12, 13, 10, 11, 8, 9, 7, 0, 1, 2, 3, 4, 5, 6,},
             /*Float*/{14, 15, 12, 13, 10, 11, 8, 9, 7, 1, 0, 2, 3, 4, 5, 6,},
             /*double*/{14, 15, 12, 13, 10, 11, 8, 9, 7, 5, 6, 0, 1, 2, 3, 4,},
@@ -434,8 +440,23 @@ public class MetaClassHelper {
         return ret;
     }
 
-    public static String capitalize(String property) {
-        return property.substring(0, 1).toUpperCase() + property.substring(1, property.length());
+    /**
+     * This is the complement to the java.beans.Introspector.decapitalize(String) method.
+     * We handle names that begin with an initial lowerCase followed by upperCase specially
+     * (which is to make no change).
+     * See GROOVY-3211.
+     * @param property the property name to capitalize
+     * @return the name capitalized, except when we don't
+     */
+    public static String capitalize(final String property) {
+        final String rest = property.substring(1);
+        
+        // Funky rule so that names like 'pNAME' will still work.
+        if (Character.isLowerCase(property.charAt(0)) && (rest.length() > 0) && Character.isUpperCase(rest.charAt(0))) {
+           return property;
+        }
+        
+        return property.substring(0, 1).toUpperCase() + rest;
     }
 
     /**
@@ -449,7 +470,7 @@ public class MetaClassHelper {
         final Object[] data = methods.getArray();
         for (int i = 0; i != len; ++i) {
             Object method = data[i];
-            final ParameterTypes pt = getParameterTypes(method);
+            final ParameterTypes pt = (ParameterTypes) method;
             CachedClass[] paramTypes = pt.getParameterTypes();
             int paramLength = paramTypes.length;
             if (paramLength == 0) {
@@ -477,7 +498,7 @@ public class MetaClassHelper {
         for (int i = 0; i != len; ++i) {
             final Object[] data = methods.getArray();
             Object method = data[i];
-            final ParameterTypes pt = getParameterTypes(method);
+            final ParameterTypes pt = (ParameterTypes) method;
             CachedClass[] paramTypes = pt.getParameterTypes();
             int paramLength = paramTypes.length;
             if (paramLength == 0 || paramLength > 2) continue;
@@ -588,37 +609,6 @@ public class MetaClassHelper {
         return ans;
     }
 
-    public static Object doConstructorInvoke(CachedConstructor constructor, Object[] argumentArray) {
-        final Constructor constr = constructor.cachedConstructor;
-        argumentArray = constructor.coerceArgumentsToClasses(argumentArray);
-        try {
-            return constr.newInstance(argumentArray);
-        } catch (InvocationTargetException e) {
-            throw new InvokerInvocationException(e);
-        } catch (IllegalArgumentException e) {
-            throw createExceptionText("failed to invoke constructor: ", constr, argumentArray, e, false);
-        } catch (IllegalAccessException e) {
-            throw createExceptionText("could not access constructor: ", constr, argumentArray, e, false);
-        } catch (Exception e) {
-            throw createExceptionText("failed to invoke constructor: ", constr, argumentArray, e, true);
-        }
-    }
-
-    private static GroovyRuntimeException createExceptionText(String init, Constructor constructor, Object[] argumentArray, Throwable e, boolean setReason) {
-        throw new GroovyRuntimeException(
-                init
-                        + constructor
-                        + " with arguments: "
-                        + InvokerHelper.toString(argumentArray)
-                        + " reason: "
-                        + e,
-                setReason ? e : null);
-    }
-
-    public static Object doMethodInvoke(Object object, MetaMethod method, Object[] argumentArray) {
-        return method.doMethodInvoke(object, argumentArray);
-    }
-
     public static Object makeCommonArray(Object[] arguments, int offset, Class fallback) {
         // arguments.leght>0 && !=null
         Class baseClass = null;
@@ -684,21 +674,6 @@ public class MetaClassHelper {
         return new MethodClosure(object, methodName);
     }
 
-    public static ParameterTypes getParameterTypes(Object methodOrConstructor) {
-        if (methodOrConstructor instanceof ParameterTypes) {
-            return (ParameterTypes) methodOrConstructor;
-        }
-        if (methodOrConstructor instanceof Method) {
-            Method method = (Method) methodOrConstructor;
-            return CachedMethod.find(method);
-        }
-        if (methodOrConstructor instanceof Constructor) {
-            Constructor constructor = (Constructor) methodOrConstructor;
-            return CachedConstructor.find(constructor);
-        }
-        throw new IllegalArgumentException("Must be a Method or Constructor");
-    }
-
     public static boolean isAssignableFrom(Class classToTransformTo, Class classToTransformFrom) {
         if (classToTransformTo == classToTransformFrom) return true;
         if (classToTransformFrom == null) return true;
@@ -790,10 +765,6 @@ public class MetaClassHelper {
         return true;
     }
 
-    public static boolean isValidMethod(Object method, Class[] arguments) {
-        return getParameterTypes(method).isValidMethod(arguments);
-    }
-
     public static void logMethodCall(Object object, String methodName, Object[] arguments) {
         String className = getClassName(object);
         String logname = "methodCalls." + className + "." + methodName;
@@ -863,4 +834,173 @@ public class MetaClassHelper {
         return wrappedArguments;
     }
 
+    public static boolean sameClasses(Class[] params, Object[] arguments, boolean weakNullCheck) {
+        if (params.length != arguments.length)
+          return false;
+
+        for (int i = params.length-1; i >= 0; i--) {
+            Object arg = arguments[i];
+            if (arg == null) {
+                if (!weakNullCheck)
+                  return false;
+            } else {
+                if (params[i] != arg.getClass()
+                     &&(!(arg instanceof Wrapper) || params[i] != ((Wrapper)arg).getType()))
+                      return false;
+            }
+        }
+
+        return true;
+    }
+
+    public static boolean sameClasses(Class[] params, Object[] arguments) {
+        if (params.length != arguments.length)
+          return false;
+
+        for (int i = params.length-1; i >= 0; i--) {
+            Object arg = arguments[i];
+            if (arg == null) {
+               if (params [i] != null)
+                 return false;
+            }
+            else {
+               if(params[i] != arg.getClass() && !(arg instanceof Wrapper && params[i] == ((Wrapper)arg).getType()))
+                 return false;
+            }
+        }
+
+        return true;
+    }
+
+    public static boolean sameClasses(Class[] params) {
+        if (params.length != 0)
+          return false;
+
+        return true;
+    }
+
+    public static boolean sameClasses(Class[] params, Object arg1) {
+        if (params.length != 1)
+          return false;
+
+        if (arg1 == null
+                || (params[0] != arg1.getClass()
+                   &&(    !(arg1 instanceof Wrapper)
+                       || params[0] != ((Wrapper) arg1).getType())))
+          return false;
+
+        return true;
+    }
+
+    public static boolean sameClasses(Class[] params, Object arg1, Object arg2) {
+        if (params.length != 2)
+          return false;
+
+        if (arg1 == null
+                || (params[0] != arg1.getClass()
+                   &&(    !(arg1 instanceof Wrapper)
+                       || params[0] != ((Wrapper)arg1).getType())))
+          return false;
+
+        if (arg2 == null
+                || (params[1] != arg2.getClass()
+                   &&(    !(arg2 instanceof Wrapper)
+                       || params[1] != ((Wrapper)arg2).getType())))
+          return false;
+
+        return true;
+    }
+
+    public static boolean sameClasses(Class[] params, Object arg1, Object arg2, Object arg3) {
+        if (params.length != 3)
+          return false;
+
+        if (arg1 == null
+                || (params[0] != arg1.getClass()
+                   &&(    !(arg1 instanceof Wrapper)
+                       || params[0] != ((Wrapper)arg1).getType())))
+          return false;
+
+        if (arg2 == null
+                || (params[1] != arg2.getClass()
+                   &&(    !(arg2 instanceof Wrapper)
+                       || params[1] != ((Wrapper)arg2).getType())))
+          return false;
+
+        if (arg3 == null
+                || (params[2] != arg3.getClass()
+                   &&(    !(arg3 instanceof Wrapper)
+                       || params[2] != ((Wrapper)arg3).getType())))
+          return false;
+
+        return true;
+    }
+
+    public static boolean sameClasses(Class[] params, Object arg1, Object arg2, Object arg3, Object arg4) {
+        if (params.length != 4)
+          return false;
+
+        if (arg1 == null
+                || (params[0] != arg1.getClass()
+                   &&(    !(arg1 instanceof Wrapper)
+                       || params[0] != ((Wrapper)arg1).getType())))
+          return false;
+
+        if (arg2 == null
+                || (params[1] != arg2.getClass()
+                   &&(    !(arg2 instanceof Wrapper)
+                       || params[1] != ((Wrapper)arg2).getType())))
+          return false;
+
+        if (arg3 == null
+                || (params[2] != arg3.getClass()
+                   &&(    !(arg3 instanceof Wrapper)
+                       || params[2] != ((Wrapper)arg3).getType())))
+          return false;
+
+        if (arg4 == null
+                || (params[3] != arg4.getClass()
+                   &&(    !(arg4 instanceof Wrapper)
+                       || params[3] != ((Wrapper)arg4).getType())))
+          return false;
+
+        return true;
+    }
+
+    public static boolean sameClass(Class[] params, Object arg) {
+        return !(arg == null
+                || (params[0] != arg.getClass()
+                && (!(arg instanceof Wrapper)
+                || params[0] != ((Wrapper) arg).getType())));
+
+    }
+
+    public static Class[] castArgumentsToClassArray(Object[] argTypes) {
+        if (argTypes == null) return EMPTY_CLASS_ARRAY;
+        Class[] classes = new Class[argTypes.length];
+        for (int i = 0; i < argTypes.length; i++) {
+            Object argType = argTypes[i];
+            if (argType instanceof Class) {
+                classes[i] = (Class) argType;
+            } else if (argType == null) {
+                classes[i] = null;
+            } else {
+//                throw new IllegalArgumentException("Arguments to method [respondsTo] must be of type java.lang.Class!");
+                classes[i] = argType.getClass();
+            }
+        }
+        return classes;
+    }
+
+    public static void unwrap(Object[] arguments) {
+        //
+        // Temp code to ignore wrapped parameters
+        // The New MOP will deal with these properly
+        //
+        for (int i = 0; i != arguments.length; i++) {
+            if (arguments[i] instanceof Wrapper) {
+                arguments[i] = ((Wrapper) arguments[i]).unwrap();
+            }
+        }
+    }
 }
