@@ -15,8 +15,8 @@
  */
 package org.codehaus.groovy.vmplugin;
 
-import org.codehaus.groovy.ast.ClassNode;
-import org.codehaus.groovy.ast.CompileUnit;
+import org.codehaus.groovy.ast.*;
+import java.lang.reflect.Method;
 
 /**
  * Interface to access VM version based actions.
@@ -26,8 +26,8 @@ import org.codehaus.groovy.ast.CompileUnit;
  */
 public interface VMPlugin {
 
-    void setGenericsTypes(ClassNode c);
+    void setAdditionalClassInformation(ClassNode c);
     Class[] getPluginDefaultGroovyMethods();
-
-    void configureClassNode(CompileUnit compileUnit, ClassNode classNode);
+    public void configureAnnotation(AnnotationNode an);
+    public void configureClassNode(CompileUnit compileUnit, ClassNode classNode);
 }
