@@ -3,8 +3,6 @@ package org.codehaus.groovy.ast;
 import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Map;
-
 import org.codehaus.groovy.ast.expr.ArgumentListExpression;
 import org.codehaus.groovy.ast.expr.ArrayExpression;
 import org.codehaus.groovy.ast.expr.AttributeExpression;
@@ -210,10 +208,10 @@ class LineCheckVisitor extends ClassCodeVisitorSupport {
 	}
 
 	public void visitAnnotations(AnnotatedNode node) {
-        Map annotionMap = node.getAnnotations();
+        List annotionMap = node.getAnnotations();
         if (annotionMap.isEmpty()) return;
 		visitNode(node);
-        Iterator it = annotionMap.values().iterator(); 
+        Iterator it = annotionMap.iterator(); 
         while (it.hasNext()) {
             AnnotationNode an = (AnnotationNode) it.next();
             visitNode(an);
