@@ -1,5 +1,5 @@
 /*
- $Id: FileSystemCompilerTest.java 11889 2008-04-21 00:18:37Z shemnon $
+ $Id: FileSystemCompilerTest.java 14140 2008-11-24 22:24:54Z shemnon $
 
  Copyright 2003 (C) James Strachan and Bob Mcwhirter. All Rights Reserved.
 
@@ -59,7 +59,7 @@ import java.util.ArrayList;
  * Tests the compiling & running of GroovyTestCases
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 11889 $
+ * @version $Revision: 14140 $
  */
 public class FileSystemCompilerTest extends GroovyTestCase {
 
@@ -101,5 +101,17 @@ public class FileSystemCompilerTest extends GroovyTestCase {
         compiler = new FileSystemCompiler(configuration);
     }
 
-}
+    public void testCommandLine() throws Exception {
+        try {
+            FileSystemCompiler.commandLineCompile(new String[] {"--bogus-option"});
+            fail("Compiler fails to reject bogus command line");
+        } catch (Exception re) {
+            // this is why shouldFail {} exists in Groovy tests
+        }
 
+        File dir = new File("target/test-generated-classes/cl");
+        dir.mkdirs();
+        FileSystemCompiler.commandLineCompile(new String[] {"src/test/groovy/LittleClosureTest.groovy", "-d", dir.getPath()});
+    }
+
+}
